@@ -6,6 +6,8 @@ use CPANPLUS::Error;
 use CPANPLUS::Internals::Constants;
 
 use File::Spec                  ();
+use File::Path                  ();
+use File::Temp                  ();
 use File::Basename              ();
 use Archive::Extract;
 use IPC::Cmd                    qw[run];
@@ -14,7 +16,7 @@ use Module::Load::Conditional   qw[can_load check_install];
 use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 
 use vars qw[$VERSION];
-$VERSION = "0.9154";
+$VERSION = "0.9158";
 
 local $Params::Check::VERBOSE = 1;
 
@@ -150,6 +152,9 @@ sub _extract {
                         $conf->_get_build('moddir'),
                 );
 
+    File::Path::mkpath( $to ) unless -d $to;
+    $to = File::Temp::tempdir( DIR => $to, CLEANUP => 0 );
+
     msg(loc("Extracting '%1'", $mod->package), $verbose);
     ### delegate to Archive::Extract ###
     ### set up some flags for archive::extract ###
