@@ -13,7 +13,7 @@ use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';
 use version;
 
 use vars qw[$VERSION];
-$VERSION = "0.9154";
+$VERSION = "0.9158";
 
 local $Params::Check::VERBOSE = 1;
 
@@ -505,6 +505,75 @@ For example, C<Foo-Bar-1.2.tar.gz> would return the following parts:
 
 =cut
 
+sub _distname_info {
+  my $file = shift or return;
+
+  my ($dist, $version) = $file =~ /^
+    ((?:[-+.]*(?:[A-Za-z0-9]+|(?<=\D)_|_(?=\D))*
+     (?:
+  [A-Za-z](?=[^A-Za-z]|$)
+  |
+  \d(?=-)
+     )(?<![._-][vV])
+    )+)(.*)
+  $/xs or return ($file,undef,undef);
+
+  if ($dist =~ /-undef\z/ and ! length $version) {
+    $dist =~ s/-undef\z//;
+  }
+
+  # Remove potential -withoutworldwriteables suffix
+  $version =~ s/-withoutworldwriteables$//;
+
+  if ($version =~ /^(-[Vv].*)-(\d.*)/) {
+
+    # Catch names like Unicode-Collate-Standard-V3_1_1-0.1
+    # where the V3_1_1 is part of the distname
+    $dist .= $1;
+    $version = $2;
+  }
+
+  if ($version =~ /(.+_.*)-(\d.*)/) {
+      # Catch names like Task-Deprecations5_14-1.00.tar.gz where the 5_14 is
+      # part of the distname. However, names like libao-perl_0.03-1.tar.gz
+      # should still have 0.03-1 as their version.
+      $dist .= $1;
+      $version = $2;
+  }
+
+  # Normalize the Dist.pm-1.23 convention which CGI.pm and
+  # a few others use.
+  $dist =~ s{\.pm$}{};
+
+  $version = $1
+    if !length $version and $dist =~ s/-(\d+\w)$//;
+
+  $version = $1 . $version
+    if $version =~ /^\d+$/ and $dist =~ s/-(\w+)$//;
+
+  if ($version =~ /\d\.\d/) {
+    $version =~ s/^[-_.]+//;
+  }
+  else {
+    $version =~ s/^[-_]+//;
+  }
+
+  my $dev;
+  if (length $version) {
+    if ($file =~ /^perl-?\d+\.(\d+)(?:\D(\d+))?(-(?:TRIAL|RC)\d+)?$/) {
+      $dev = 1 if (($1 > 6 and $1 & 1) or ($2 and $2 >= 50)) or $3;
+    }
+    elsif ($version =~ /\d\D\d+_\d/ or $version =~ /-TRIAL/) {
+      $dev = 1;
+    }
+  }
+  else {
+    $version = undef;
+  }
+
+  ($dist, $version, $dev);
+}
+
 {   my $del_re = qr/[-_\+]/i;           # delimiter between elements
     my $pkg_re = qr/[a-z]               # any letters followed by
                     [a-z\d]*            # any letters, numbers
@@ -575,6 +644,18 @@ sub _split_package_string {
         my $tmpl = { package => { required => 1, store => \$str } };
         check( $tmpl, \%hash ) or return;
 
+        my ($dpkg,$dver);
+        {
+          my ($base,$ext);
+          if ( $str =~ m,([^/]+)\.(tar\.(?:[gx]?z|bz2)|zip|tbz|tgz|txz)$,i ) {
+             $base = $1;
+             $ext  = $2;
+          }
+          else {
+            $base = $str;
+          }
+          ($dpkg,$dver) = _distname_info($base);
+        }
 
         ### 2 different regexes, one for the 'perl' package,
         ### one for ordinary CPAN packages.. try them both,
@@ -596,6 +677,9 @@ sub _split_package_string {
             ### strip the .pm package suffix some authors insist on adding
             $pkg =~ s/\.pm$//i;
 
+            $pkg = $dpkg if $dpkg && $pkg ne $dpkg;
+            $ver = $dver if $dver && $ver ne $dver;
+
             return ($pkg, $ver, $ext, $full );
         }
 
